#include "verbimorfo.hh"
#include "nominimorfo.hh"
#include "argh.hh"

#include <string>
#include <cstdio>
using std::string;
using std::printf;
using std::fprintf;

int main(int argc, const char *const *argv)
{
    bool nega = false;
    string pers = "pas";
    string aika = "preesens";
    string tapa = "pos";
    string sija = "nom";
    string medipre = "";
    bool monikko = false;
    bool passiivi = false;
    int verbose = 0;
    
    ParamHandler Argh;
    Argh.AddLong("help",   'h').SetBool().SetDesc("This help.");
    Argh.AddLong("version",'V').SetBool().SetDesc("Displays version information.");
    Argh.AddLong("pers",   'p').SetString().SetDesc("Valitse persoonamuoto/omistusliite (yks1,yks2,yks3,mon1,mon2,mon3,pass)", "<muoto>");
    Argh.AddLong("nega",   'n').SetBool().SetDesc("Valitse negaatio (finiittien verbien taivutukseen)");
    Argh.AddLong("moni",   'l').SetBool().SetDesc("Valitse monikko (nominien ja nominaalien taivutukseen)");
    Argh.AddLong("pass",   'a').SetBool().SetDesc("Valitse passiivi (partisiippien ja III infinitiivin taivutukseen)");
    Argh.AddLong("aika",   't').SetString().SetDesc("Valitse aikamuoto (pr,i,pe,pl finiittien verbien taivutukseen)", "<aika>");
    Argh.AddLong("tapa",   'm').SetString().SetDesc("Valitse tapaluokka (ind,kond,pot,imp,[1-5]i,1p,2p,suor,johto[1-6]) (verbin taivutukseen)", "<tapa>");
    Argh.AddLong("sija",   'j').SetString().SetDesc("Valitse sijamuoto (nom,gen,ak,es,pa,tr,in,el,il,ade,abl,al,abe,kom,ins)", "<sija>");
    Argh.AddLong("medipre",'M').SetString().SetDesc("Mediaalinen esiksittely verbeille (a/p)", "<string>");
    Argh.AddLong("verbose",'v').SetBool().SetDesc("Verbose (cumulative)");
    
    Argh.StartParse(argc, argv);
    bool sanoja = false;
    int c;
    for(; (c=Argh.GetParam()) != -1; ++c)
    {
        switch(c)
        {
            case 'V': printf("%s\n", VERSION); return 0;
            case 'p': pers = Argh.GetString(); break;
            case 'n': nega = Argh.GetBool(); break;
            case 't': aika = Argh.GetString(); break;
            case 'm': tapa = Argh.GetString(); break;
            case 'j': sija = Argh.GetString(); break;
            case 'M': medipre = Argh.GetString(); break;
            case 'l': monikko = Argh.GetBool(); break;
            case 'a': passiivi = Argh.GetBool(); break;
            case 'v': if(Argh.GetBool())++verbose;else --verbose; break;
            case 'h':
                if(Argh.GetBool())
                {
            Help:   printf(
                        "formodemo v"VERSION" - Copyright (C) 1992,2001 Bisqwit (http://iki.fi/bisqwit/)\n"
                        "Thanks to Warp :)\n"
                        "\n"
                        "Usage: %s [<option> [<...>]] <word> [<...>]\n"
                        "\n"
                        "Options:\n", argv[0]);
                    Argh.ListOptions();
                    printf(
                    	"\nNote: Give options first, words then.\n"
                    	"New options affect only words next to them.\n");                    	
                    return 0;
                }
                break;
            default:
            {
                string word = Argh.GetString();
                
                for(unsigned m=0; m<medipre.size(); ++m)
                {
                	if(medipre[m]=='a')
                	    word = Mediaaliksi(word, false);
                	else if(medipre[m]=='p')
                	    word = Mediaaliksi(word, true);
                	else
                	{
                	    fprintf(stderr, "%s: Unknown medial key: '%c' ('a' and 'p' are known)\n",
                	        word.c_str(), medipre[m]);
                	}
                }

                string result;
                
                aikamuoto a;
                sijamuoto j;
                persoona p;
                tapaluokka t;
                
/* The following code has been generated by hashes from hashes.txt */
#include "mainhash.inc"

                if(nega && t != INDIKATIIVI && t != KONDITIONAALI && t != POTENTIAALI && t != IMPERATIIVI)
                {
                    fprintf(stderr, "%s: Negaatio soveltuu vain finiitteihin verbeihin. Kytmme siis indikatiivia.\n", word.c_str());
                    t = INDIKATIIVI;
                }
                if(passiivi && t != PARTISIIPPI1 && t != PARTISIIPPI2 && t != INFINITIIVI3)
                {
                    fprintf(stderr, "%s: -a (passiivi) -vipu soveltuu vain partisiippeihin ja III infinitiiviin.\n"
                                    "Jos haluat saada muunlaisen verbin passiivimuotoon, valitse passiivinen persoona -ppass -vivulla.\n", word.c_str());
                }
                if(j != NOMINATIIVI && (t == INDIKATIIVI || t == KONDITIONAALI || t == POTENTIAALI || t == IMPERATIIVI))
                {
                    fprintf(stderr, "%s: Finiiteill verbeill ei ole sijamuotoja. Kytmme siis IV infinitiivi.\n", word.c_str());
                    t = INFINITIIVI4;
                }
                sanoja = true;
                switch(t)
                {
                    case INDIKATIIVI:      printf("Indikatiivi: '%s'\n", Indikatiiviksi(word,a,p,!nega).c_str()); break;
                    case KONDITIONAALI:    printf("Konditionaali: '%s'\n", Konditionaaliksi(word,a,p,!nega).c_str()); break;
                    case POTENTIAALI:      printf("Potentiaali: '%s'\n", Potentiaaliksi(word,a,p,!nega).c_str()); break;
                    case IMPERATIIVI:      printf("Imperatiivi: '%s'\n", Imperatiiviksi(word,a,p,!nega).c_str()); break;
                    case PARTISIIPPI1:     printf("I partisiippi: '%s'\n", NominiTaivutus(Partisiipiksi1(word,passiivi),j,monikko,p).c_str()); break;
                    case PARTISIIPPI2:     printf("II partisiippi: '%s'\n", NominiTaivutus(Partisiipiksi2(word,passiivi),j,monikko,p).c_str()); break;
                    case INFINITIIVI1:     printf("I infinitiivi: '%s'\n", InfiniTaivutus(1, Infinitiiviksi1(word), j, monikko, p).c_str()); break;
                    case INFINITIIVI2:     printf("II infinitiivi: '%s'\n", InfiniTaivutus(2, Infinitiiviksi2(word), j, monikko, p).c_str()); break;
                    case INFINITIIVI3:     printf("III infinitiivi: '%s'\n", InfiniTaivutus(3, Infinitiiviksi3(word,passiivi), j, monikko, p).c_str()); break;
                    case INFINITIIVI4:     printf("IV infinitiivi: '%s'\n", InfiniTaivutus(4, Infinitiiviksi4(word), j, monikko, p).c_str()); break;
                    case INFINITIIVI5:     printf("V infinitiivi: '%s'\n", InfiniTaivutus(5, Infinitiiviksi5(word), j, monikko, p).c_str()); break;
                    case JOHTO1:
                    case JOHTO2:
                    case JOHTO3:
                    case JOHTO4:
                    case JOHTO5:
                    case JOHTO6:           printf("Johdos: '%s'\n", NominiTaivutus(JohdaVerbi(word, t), j, monikko, p).c_str()); break;
                    case SUORITTAJA:       printf("Suorittaja: '%s'\n", NominiTaivutus(Suorittajaksi(word), j, monikko, p).c_str()); break;
                    case POSITIIVI:        printf("Positiivi: '%s'\n", NominiTaivutus(word, j, monikko, p).c_str()); break;
                    case KOMPARATIIVI:     printf("Komparatiivi - EI TOIMI VIEL: '%s'\n", NominiTaivutus(word, j, monikko, p).c_str()); break;
                    case SUPERLATIIVI:     printf("Superlatiivi - EI TOIMI VIEL: '%s'\n", NominiTaivutus(word, j, monikko, p).c_str()); break;
                    default: sanoja = false;
                }
            }
        }
    }
    if(Argh.ok() && !sanoja)
    {
        fprintf(stderr, "Sanaton!\n");
        goto Help;
    }
    return 0;
}
